﻿--[[
	LootMaster, Master looter stuff
]]--

local debug	= false

LootMasterML	    = LibStub("AceAddon-3.0"):NewAddon("LootMasterML", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0")
local LootMaster    = LibStub("AceAddon-3.0"):GetAddon("LootMaster")

local addon         = LootMasterML		-- Local instance of the addon

function LootMasterML:Debug( message )
    if not LootMaster or not LootMaster.debug then return end;
    self:Print("debug: " .. message)
end

function LootMasterML:OnInitialize()

	self.lootTable = {}			-- a table to store loot info

	-- Event Register
	--self:RegisterEvent("LOOT_OPENED")
	--self:RegisterEvent("LOOT_SLOT_CLEARED")	

	self:RegisterEvent("OPEN_MASTER_LOOT_LIST")

    -- Clear the item cache on ungroup. -- should really not be necessary, change this in the future
	--self:RegisterEvent("PARTY_MEMBERS_CHANGED", 	"ClearCacheOnUngroup")
    
    self:RegisterEvent("CHAT_MSG_LOOT");
    self:RegisterEvent("PLAYER_REGEN_DISABLED", "EnterCombat");
    self:RegisterEvent("PLAYER_REGEN_ENABLED", "LeaveCombat");
    
    self:RegisterEvent("GUILD_ROSTER_UPDATE", "CacheGuildInfo")
    

	-- Register communications
	self:RegisterComm("EPGPLootMasterML", 		    "CommandReceived")
    
end

function LootMasterML:GetVersionString()
    local lm = LibStub("AceAddon-3.0"):GetAddon("LootMaster")
    return (lm.GetVersionString(lm) or 'unknown version')
end

function LootMasterML:SendCommand(command, message, target)
	if not target then
		return self:Print("Could not send command, no target specified")
	end;
    if target=='RAID' then
        self:SendCommMessage("EPGPLootMasterC", format("%s:%s", tostring(command), tostring(message)), "RAID")
    elseif target=='PARTY' then
        self:SendCommMessage("EPGPLootMasterC", format("%s:%s", tostring(command), tostring(message)), "PARTY")
    else
        self:SendCommMessage("EPGPLootMasterC", format("%s:%s", tostring(command), tostring(message)), "WHISPER", target)
    end
end

--[[
	Event gets triggered when ML receives a message from a candidate
]]
function LootMasterML:CommandReceived(prefix, message, distribution, sender)

	local _,_,command, message = string.find(message, "^([%a_]-):(.*)$")
	command = strupper(command or '');
	message = message or '';
	
	if command == 'WANT' then
        
		-- A candidate just told us he wants the item, lets just give it to him for testing purposes
        local _,_, itemID, response, enchantingSkill = string.find(message, "^([^\^]-)\^([^\^]-)\^(.*)$");
        enchantingSkill = tonumber(enchantingSkill) or 0;
        response = tonumber(response)
        
        if response == LootMaster.RESPONSE.PASS and enchantingSkill~=0 then
            response = LootMaster.RESPONSE.DISENCHANT;
        end
        
        if self:IsCandidate( itemID, sender ) then        
            self:SetCandidateData( itemID, sender, 'response', response );
            self:SetCandidateData( itemID, sender, 'enchantingSkill', enchantingSkill );     
            self:ReloadMLTableForLoot( itemID )
        else
            self:Print( format("%s responded WANT for %s but loot or candidate not found (Already looted?)", sender or 'nil', itemID or 'nil'))
        end
		
	elseif command == 'GEAR' then

		-- A candidate just sent us his current gear. update the cache
        local _,_,itemID, iVersion, gear = string.find(message, "^([^\^]-)\^([^\^]-)\^(.*)$");
        local item1, item2 = strsplit('$', gear)
        
        if self:IsCandidate( itemID, sender ) then        
            self:SetCandidateData( itemID, sender, 'currentitem', item1 );
            self:SetCandidateData( itemID, sender, 'currentitem2', item2 );
            self:SetCandidateData( itemID, sender, 'version', iVersion );
            self:SetCandidateData( itemID, sender, 'response', LootMaster.RESPONSE.WAIT );
            self:ReloadMLTableForLoot( itemID )
        else
            self:Print( format("%s responded GEAR for %s but loot or candidate not found (Already looted?)", sender or 'nil', itemID or 'nil'))
        end
        
    else
		self:Print( format("MLRCV(%s): %s", tostring(command), tostring(message) ) )
	end
end

local deformat_cache = {}
function LootMasterML:Deformat(str, format)
  local pat = deformat_cache[format]
  if not pat then
    -- Escape special characters
    pat = format:gsub("[%^%$%(%)%%%.%[%]%*%+%-%?]",
                          function(c) return "%"..c end)
    pat = pat:gsub("%%%%([sd])", {
                             ["s"] = "(.-)",
                             ["d"] = "(%d+)",
                           })
    deformat_cache[format] = pat
  end
  return str:match(pat)
end

--[[
	Kind of singleton, just create the frame once.
]]
function LootMasterML:GetMasterLootFrame()
	if self.masterframe then return self.masterframe end
    --  self.masterframe

	

	return self.masterframe;
end

--[[
	Send a request to the specified player if he/she is interested in the given loot
]]
function LootMasterML:AskCandidateIfNeeded( link, candidate )

	local loot = self:GetLoot( link );
	if not loot then
		return self:Print( format("Could not ask player if needed because %s is not cached", link) )
	end
    
    -- Hardcoded time in secs until autopass
    self.timeout = LootMaster.db.profile.loot_timeout;

	self:SendCommand( "DO_YOU_WANT", format('%s^%s^%s^%s^%s^%s^%s^%s^%s^%s', loot.id or '', loot.gpvalue or -1, loot.ilevel or -1, loot.gpvalue2 or -1, loot.binding or '', loot.equipLoc or 0, loot.quality or 0, self.timeout or 60, loot.link, loot.texture or ''), candidate )

    self:SetCandidateData( link, candidate, 'response', LootMaster.RESPONSE.INIT );

	-- Update the ui
	self:ReloadMLTableForLoot( loot.id )
end

--[[
	Add loot to masterloot cache. This is where candidate responses are stored, plus the rows for the scrollingtable
]]
function LootMasterML:AddLoot( link )
	if not link then return end;
	if not self.lootTable then self.lootTable={} end;	

	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(link)
    local _,_,itemID = strfind(itemLink, 'Hitem:(%d+)');
    if not itemID or not itemName then return end;

	if self.lootTable[itemID] then return itemID end;

	-- Calc the EPGP values for this item
	local gpvalue, gpvalue2, ilevel = GetGPValue( itemLink );

	-- See if the item is BoP, BoE or BoU
	local itemBind = LootMaster:GetItemBinding( itemLink )	

	self.lootTable[itemID] = {
		['link']	        = itemLink,
		['name']	        = itemName,
        
        ['id']              = itemID,
        ['itemID']          = itemID,
        ['itemid']          = itemID,

		['gpvalue']	        = gpvalue or '',
        ['gpvalue_manual']	= gpvalue or '',	
		['ilevel']	        = ilevel or 0,
		['binding']	        = itemBind,
        ['quality']         = itemRarity or 0,
        
        ['texture']         = itemTexture or '',
		
		['equipLoc']	    = itemEquipLoc or '',
		
		['started']	        = nil,

		['rowdata']	        = {},
		['candidates']	    = {}
	}

	return itemID;
end

--[[
	Get the lootdata from masterloot cache. This is where candidate responses are stored, plus the rows for the scrollingtable. Returns nil if not found
]]
function LootMasterML:GetLoot( link )
	if not link then self:Debug('getloot: !link'); return nil end;
	if not self.lootTable then self:Debug('getloot: !lootTable'); return nil end;	
    
    if self.lootTable[link] then return self.lootTable[link] end;
    
    local _,_,itemID = strfind(link, 'Hitem:(%d+)');
    if not itemID then self:Debug('getloot: !itemID: ' .. tostring(itemID)); return end;
    if not self.lootTable[itemID] then self:Debug('getloot: !lootTable[itemID]: ' .. tostring(itemID)); return end;

	return self.lootTable[itemID];
end

--[[
	Removes the given loot from the masterloot cache
]]
function LootMasterML:RemoveLoot( link )
	if not link then return end;
	if not self.lootTable then return end;	
    
    if self.lootTable[link] then 
        self.lootTable[link] = nil;
        if self.frame and self.frame.currentLoot and self.frame.currentLoot.itemID == link then
            self.frame.currentLoot = nil;
        end
        return true
    end
    
    local _,_,itemID = strfind(link, 'Hitem:(%d+)');
    if not itemID or not self.lootTable[itemID] then return self:Debug(format('RemoveLoot: not found %s %s(%s)', link or 'nil', itemID or 'nil', type(itemID))) end;
    
    if self.frame and self.frame.currentLoot and self.frame.currentLoot.itemID == itemID then
        self.frame.currentLoot = nil;
    end

    self:Debug('RemoveLoot: '..link)

	self.lootTable[itemID] = nil;
	return true;
end

function LootMasterML:GetEPGP( player )
    if not EPGP or not EPGP.GetEPGP then return nil, nil, nil end;
    local ret, ep, gp, alt = pcall(EPGP.GetEPGP, EPGP, player)
    if not ret then return nil, nil, nil end;
    return ep, gp, alt;
end

function LootMasterML:GetEP( player )
    local ep, gp = self:GetEPGP( player );
    if not ep then return '?' end;
    return format("%.4g", ep or 0)
end

function LootMasterML:GetGP( player )
    local ep, gp = self:GetEPGP( player );
    if not gp then return '?' end;
    return format("%.4g", gp or 0)
end

function LootMasterML:GetPR( player )
    local ep, gp = self:GetEPGP( player );
    if not gp or not ep then return '?' end;
    if gp>0 then return format("%.4g", ep/gp) end;
    return 0;
end

--[[
	Adds a candidate (player eligible to receive loot) to the masterloot cache
	for the given loot
]]
function LootMasterML:AddCandidate( loot, candidate )
	local itemID = self:AddLoot( loot )
	if not itemID then return end
	if not candidate then return end
	if not self.lootTable[itemID] then return end

	-- Just return the itemname if the candidate already exists
	if self.lootTable[itemID]['candidates'][candidate] then return itemID end;
    
    math.randomseed(math.random(0,2147483647)+(GetTime()*1000));
    
    local guildRankName = ''
    if self.guildInfo and self.guildInfo[candidate] then
        guildRankName = self.guildInfo[candidate]
    end
    


	tinsert( self.lootTable[itemID]['rowdata'],
		 {  ["onclick"]         = function(arg1, button, down) if button=='RightButton' then self:OnCandidateRowRightClick(candidate, loot, arg1) end end,
			["cols"] = {
				{["name"]       = 'name',
				 ["value"]      = candidate,
                 ["color"]      = self.GetCandidateCellColor,
                 ["colorargs"]  = {self, candidate, itemID}},
                 
                {["name"]       = 'guildrank',
				 ["value"]      = guildRankName,
                 ["color"]      = self.GetCandidateCellColor,
                 ["colorargs"]  = {self, candidate, itemID}},  
                 
                {["name"]       = 'response',
				 ["value"]      = LootMaster.RESPONSE.NOTANNOUNCED,
                 ["args"]       = {self, candidate, itemID},
                 ["userDraw"]   = addon.SetCandidateResponseCellUserDraw,
                 ["color"]      = self.GetCandidateCellColor,
                 ["colorargs"]  = {self, candidate, itemID}},
				
				{["name"]       = 'EP',
				 ["value"]      = self.GetEP,
                 ["args"]       = {self, candidate}},
                 
                {["name"]       = 'GP',
				 ["value"]      = self.GetGP,
                 ["args"]       = {self, candidate}},
                 
                {["name"]       = 'PR',
				 ["value"]      = self.GetPR,
                 ["args"]       = {self, candidate}},
                 
                {["name"]       = 'roll',
				 ["value"]      = math.random(1, 99) + math.random(1, 10000)/10000,
                 ["userDraw"]   = addon.SetCandidateRollCellUserDraw,                 
                 ["onenter"]    = addon.ShowRollCellPopup,
                 ["onleave"]    = addon.HideGearCellPopup,
                 ["onenterargs"]= { self,candidate,itemID },
                 ["onleaveargs"]= { self,candidate,itemID },
                 ["args"]       = {self, candidate, itemID}},
                 
                {["value"]      = ' '},     -- spacer

				{["name"]       = 'currentilevel',
				 ["value"]      = '',
                 ["args"]       = {self, candidate, itemID},
                 ["userDraw"]   = addon.SetGearCelliLVL},

				{["name"]       = 'currentgp',
				 ["value"]      = '',
                 ["args"]       = {self, candidate, itemID},
                 ["userDraw"]   = addon.SetGearCellGP},

				{["name"]       = 'currentitem',
				 ["value"]      = '',
                 ["userDraw"]   = addon.SetGearCellOwnerDraw,
                 ["onenter"]    = addon.ShowGearCellPopup,
                 ["onleave"]    = addon.HideGearCellPopup,
                 ["onclick"]    = addon.OnGearCellClick,
                 ["onenterargs"]= { self,candidate,itemID,'currentitem' },
                 ["onleaveargs"]= { self,candidate,itemID,'currentitem' },
                 ["onclickargs"]= { self,candidate,itemID,'currentitem' }
                },
                
                {["name"]       = 'currentitem2',
				 ["value"]      = '',
                 ["userDraw"]   = addon.SetGearCellOwnerDraw,
                 ["onenter"]    = addon.ShowGearCellPopup,
                 ["onleave"]    = addon.HideGearCellPopup,
                 ["onclick"]    = addon.OnGearCellClick,
                 ["onenterargs"]= { self,candidate,itemID,'currentitem2' },
                 ["onleaveargs"]= { self,candidate,itemID,'currentitem2' },
                 ["onclickargs"]= { self,candidate,itemID,'currentitem2' }
                },
                
                {["value"]      = ' '},     -- spacer
			}
		 })
	
	self.lootTable[itemID]['candidates'][candidate] = #(self.lootTable[itemID]['rowdata']);

	return itemID;	
end

--[[
	Returns true if the candidate already has been cached for the given loot
]]
function LootMasterML:IsCandidate( loot, candidate )
	
    if not loot then return false end;
    
    loot = self:GetLoot(loot);    
    if not loot or not loot.id then return false end;

	if not loot
		or not self.lootTable
		or not self.lootTable[loot.id]
		or not self.lootTable[loot.id]['candidates'] then return false end;

	local rowID = self.lootTable[loot.id]['candidates'][candidate];
	if not rowID then return false end;

	return true;
end

--[[
	Stores some data in the candidate cache for the given loot
]]
function LootMasterML:SetCandidateData( loot, candidate, name, value )

	local itemID = self:AddCandidate( loot, candidate );
	if not itemID then return end;

	local rowID = self.lootTable[itemID]['candidates'][candidate];
	if not rowID then return end;

	for index,data in pairs( self.lootTable[itemID]['rowdata'][rowID]['cols'] ) do
		if data and data['name'] == name then
			self.lootTable[itemID]['rowdata'][rowID]['cols'][index]['value'] = value;
			return value
		end
	end

	self.lootTable[itemID]['rowdata'][rowID][name] = value;
	return value;
end

--[[
	Retrieves some data in the candidate cache for the given loot
]]
function LootMasterML:GetCandidateData( loot, candidate, name )

    if not loot then return end;
	loot = self:GetLoot(loot)

	if not loot or not loot.id
		or not self.lootTable
		or not self.lootTable[loot.id]
		or not self.lootTable[loot.id]['candidates'] then return end;

	local rowID = self.lootTable[loot.id]['candidates'][candidate];
	if not rowID then return end;

	for index,value in pairs( self.lootTable[loot.id]['rowdata'][rowID]['cols'] ) do
		if value and value['name'] == name then
			return self.lootTable[loot.id]['rowdata'][rowID]['cols'][index]['value']
		end
	end

	return self.lootTable[loot.id]['rowdata'][rowID][name];
end

--[[
	Tries to give the loot to the given candidate
]]
function LootMasterML:GiveLootToCandidate( link, candidate, lootType, gp )

	local candidateID = nil;
	local slotID = nil;
	local loot = self:GetLoot(link)
    
    if not loot or not loot.id then
        return self:Print( format("Could not send %s to %s, loot not found in cache", tostring(link), tostring(candidate) ) )
    end

	-- Look for the candidateID
	for cID = 1, 40 do
		local cName = GetMasterLootCandidate(cID)
		if cName and cName==candidate then
			candidateID = cID;
			break
		end;
	end;

	-- Look for the lootslotID
	for sID = 1, GetNumLootItems() do
		local _, sName = GetLootSlotInfo(sID);
		if sName and sName==loot.name then
			slotID = sID;
			break
		end;
	end;

	if not slotID then
		return self:Print( format("Could not send %s to %s, lootslotID not found (already looted or lootwindow closed?) ", tostring(loot.link), tostring(candidate) ) )
	end
	if not candidateID then
		return self:Print( format("Could not send %s to %s, candidate not found (offline, left group?)", tostring(loot.link), tostring(candidate) ) )
	end
    
    -- [[ lootType == self.LOOTTYPE.BANK, self.LOOTTYPE.DISENCHANT, self.LOOTTYPE.GP ]]--
    -- [[ gp = amount of gp awarded to this item ]] --
    
    if lootType ~= LootMaster.LOOTTYPE.GP then
        gp = 0;
    end
    
    self:SetCandidateData( loot.id, candidate, 'lootType', lootType or 0 );
    self:SetCandidateData( loot.id, candidate, 'lootGP', tonumber(gp) or 0 );

	GiveMasterLoot( slotID, candidateID )
	
end

function LootMasterML:CacheGuildInfo( event )
    self.guildInfo = {}
    
    local num = GetNumGuildMembers(false);
    for i=1, num do repeat
        local name, rank = GetGuildRosterInfo(i)
        self.guildInfo[name] = rank;
    until true end
    
end

local LootMessages = {
        ["SINGLE"]      = "([^%s]+) receives loot: (.+)\.",
	    ["SINGLESELF"]  = "You receive loot: (.+)\.",
	    ["MULTI"]       = "([^%s]+) receives loot: (.-)x(%d+)\.",
	    ["MULTISELF"]   = "You receive loot: (.-)x(%d+)\."
};

--[[ Someone received loot, lets see if it's in our cache. Send all candidates 
     a message about this, so they can update their UI and register the lootdrop
     in a raidtracker for example ]]--
function LootMasterML:CHAT_MSG_LOOT( event, message )
    
    local iStart, iEnd, sPlayer, sLink, iCount;    
    
    self:Debug("CHAT_MSG_LOOT: " .. message)
		
	if(string.find(message, LootMessages.MULTI)) then
			iStart, iEnd, sPlayer, sLink, iCount = string.find(message, LootMessages.MULTI);
            
	elseif(string.find(message, LootMessages.MULTISELF)) then
			iStart, iEnd, sLink, iCount = string.find(message, LootMessages.MULTISELF);
			sPlayer = UnitName("player");
            
    elseif(string.find(message, LootMessages.SINGLE)) then
			iStart, iEnd, sPlayer, sLink = string.find(message, LootMessages.SINGLE);
			iCount = 1;
            
	elseif(string.find(message, LootMessages.SINGLESELF)) then
			iStart, iEnd, sLink = string.find(message, LootMessages.SINGLESELF);
			iCount = 1;
			sPlayer = UnitName("player");
	end
    
    if not sPlayer or not sLink or not iCount or tonumber(iCount)>1 then
        self:Debug("!player or !link or !iCount or iCount>1");
        return
    end;
    
    self:Debug(format("%s: %sx%s", sPlayer or 'nil', sLink or 'nil', iCount or 0 ))
    
    --self:SetCandidateData( sLink, sPlayer, 'test', '0' );
    --self:SetCandidateData( sLink, sPlayer, 'lootType', LootMaster.LOOTTYPE.GP );
    
    -- ok, someone looted something... lets see if we can find it in our cache    
    local loot = self:GetLoot( sLink )
    
    -- Did we find anything?
    if not loot or not loot.candidates then return self:Debug('loot not found') end;
    
    -- We did, now get some info about the loot adressee
    local lootType = self:GetCandidateData( sLink, sPlayer, 'lootType' ) or LootMaster.LOOTTYPE.UNKNOWN;
    local lootGP = tonumber(self:GetCandidateData( sLink, sPlayer, 'lootGP' )) or -1;
    
    if lootType==LootMaster.LOOTTYPE.GP and lootGP~=0 then
        if CanEditOfficerNote() and EPGP and EPGP.IncGPBy then
            EPGP.IncGPBy(EPGP, sPlayer, sLink or 'nil', lootGP)
        else
            self:Print( format("Could not increase GP in officernotes for %s %s", sPlayer or 'unknown player', sLink or 'unknown loot') )
        end
    end;
    
    -- now send everyone in raid/party/candidates some info about the drop so they can update their ui
    if GetNumRaidMembers()>0 then
        self:Debug("send to raid");
        self:SendCommand( 'LOOTED', format('%s^%s^%s^%s', sPlayer, sLink, lootType, lootGP ), 'RAID' );
    elseif GetNumPartyMembers()>0 then
        self:Debug("send to party");
        self:SendCommand( 'LOOTED', format('%s^%s^%s^%s', sPlayer, sLink, lootType, lootGP ), 'PARTY' );
    else
        self:Debug("send to candidates");
        for candidate, id in pairs(loot.candidates) do
            self:SendCommand( 'LOOTED', format('%s^%s^%s^%s', sPlayer, sLink, lootType, lootGP ), candidate );
        end
    end
    
    -- Since it's looted, remove it from the cache and update the UI
    self:RemoveLoot(sLink);
    self:UpdateUI();
    
    self:Debug("CHAT_MSG_LOOT end");
    
end

--[[
	Event triggers when the master looter opens the popup on the loot screen
	In here we try to see what loot got selected and find the candidates for the
	loot. Ask the candidates if they want the loot. Tho, cache everything and make sure
	we don't ask the candidates more than one time (this could happen when the lootwindow
	gets opened more than one time)
]]
function LootMasterML:OPEN_MASTER_LOOT_LIST()

	-- Close the default confirm window
	StaticPopup_Hide("CONFIRM_LOOT_DISTRIBUTION");
	
	--[[
	-- Some values we probably need
	LootFrame.selectedLootButton
	LootFrame.selectedSlot
	LootFrame.selectedQuality
	LootFrame.selectedItemName
	]]

	-- local lootIcon, lootName, lootQuantity, rarity = GetLootSlotInfo(LootFrame.selectedSlot);
	local _, lootName, lootQuantity, rarity = GetLootSlotInfo(LootFrame.selectedSlot);
	local link = GetLootSlotLink(LootFrame.selectedSlot)

	-- A little sanity check; lets see if quantity == 1
	if lootQuantity~=1 then
		self:Print( format("Could not redistribute %s because quantity != 1 (%s). Please handle it manually. Create a ticket on curseforge if this happens often.", link, lootQuantity) )
		return;
	end

	-- itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture
	-- local _,_,_, itemLevel, itemMinLevel, itemType, itemSubType,_, itemEquipLoc, itemTexture = GetItemInfo(link)

	-- Register the loot in the loottable
	local lootID = self:AddLoot(link);
    if not lootID then return self:Print('Could not register loot') end;
    
    -- Auto announce?
    local autoAnnounce = rarity >= (LootMaster.db.profile.auto_announce_threshold or 4);    
    if LootMaster.db.profile.auto_announce_threshold == 0 then autoAnnounce=false end;
    
    if autoAnnounce then
        self.lootTable[lootID].announced = true;
    end

	-- Ok Lets see. Who are the candidates for this slot?
	for candidateID = 1, 40 do repeat
		local candidate = GetMasterLootCandidate(candidateID)
		if not candidate then break end;

		if not self:IsCandidate( lootID, candidate ) then

			-- Create the candidate for link;
			self:AddCandidate( lootID, candidate );

			-- Send a message to the candidate with the new loot
			if autoAnnounce then
                self:AskCandidateIfNeeded( lootID, candidate )
            end
		end	
	until true end
    
    self:ReloadMLTableForLoot( lootID )
    
    if (not autoAnnounce) and LootMaster.db.profile.auto_announce_threshold ~= 0 and LootMaster.db.profile.UseAutoLoot and LootMaster.db.profile.AutoLooter~='' then
        -- loot is below autoAnnounce threshold and UseAutoLoot is enabled.
        -- Lets see if we have a BoE or BoU item.
        
        local binding = self.lootTable[lootID].binding;
        if not binding or binding=='use' or binding=='equip' then
            -- We have an item that doesn't bind or is BoE or BoE, try to give the loot.
            
            if self:IsCandidate( lootID, LootMaster.db.profile.AutoLooter or '' ) then 
                self:Print(format('Auto looting %s to %s', link or 'nil', LootMaster.db.profile.AutoLooter or 'nil'))
                self:GiveLootToCandidate( lootID, LootMaster.db.profile.AutoLooter or '', LootMaster.LOOTTYPE.BANK)
            else
                self:Print(format('Auto looting of %s to %s failed. Not a candidate for this loot.', link or 'nil', LootMaster.db.profile.AutoLooter or 'nil'))
            end
        end
    end
    
end

-- Cache the current loot so we can reconstruct what got looted on LOOT_SLOT_CLEARED
function LootMasterML:LOOT_OPENED( event )
	self.currentLoot = {}
	
	for slotID = 1, GetNumLootItems() do repeat

		-- Does the current lootslot contain coins?
		if (LootSlotIsCoin(slotID)) then

			--[[	Ninja those coins already and move on to the next lootslot! :P
				Too bad we have to split the pot with the rest of the party/raid ^_^ ]]--
			LootSlot(slotID)
			break; -- Continue on next item in forloop, because we will break the repeat loop.
		end

		-- Else cache the lootlink for later use in LOOT_SLOT_CLEARED
		self.currentLoot[slotID] = GetLootSlotLink(slotID);

	until true end
end

-- Use the cache above and try to clear the lootTable
function LootMasterML:LOOT_SLOT_CLEARED( event, slotID )
	if not self.currentLoot or not self.currentLoot[slotID] then return end;
	local ret = self:RemoveLoot( self.currentLoot[slotID] )
end

-- We closed the lootscreen, clear the lootcache
function LootMasterML:LOOT_CLOSED( event )
	self.currentLoot = nil
end

-- Gets fired when party or raid changes
function LootMasterML:ClearCacheOnUngroup( event )
	-- Lets find out if we're still grouped, if not; then reset the cache.
	if (GetNumRaidMembers() or 0)==0 and (GetNumPartyMembers() or 0)==0 then
		self.currentLoot = nil;
		self.lootTable = nil;
	end
end








function LootMasterML:LOOT_OPENED_OLD( event, arg1, arg2, arg3, arg4 )

	--self:Print( format("LOOT_OPENED(%s, %s, %s, %s)", arg1 or 'nil', arg2 or 'nil', arg3 or 'nil', arg4 or 'nil' ))	

	-- Are we lootmaster?
	local lootmethod, masterlooterPartyID, masterlooterRaidID = GetLootMethod()
	local playerIsLootmaster = (lootmethod == 'master' and masterlooterPartyID == 0)

	-- Lets find out how many peeps in the group are able to receive the loot
	-- Per loot the number of candidates seem te same, so lets cache it.
	local candidates = {}
	for candidateID = 1, 40 do repeat
		local candidate = GetMasterLootCandidate(candidateID)
		if not candidate then break end;

		candidates[candidateID] = candidate
		self:Print( format("loot - candidate (%s) %s", candidate, candidate) )
	until true end
	local numCandidates = #candidates;

	-- Loop though all available lootslots
	for slotID = 1, GetNumLootItems() do repeat

		-- Does the current lootslot contain coins?
		if (LootSlotIsCoin(slotID)) then

			--[[	Ninja those coins already and move on to the next lootslot! :P
				Too bad we have to split the pot with the rest of the party/raid ^_^ ]]--
			LootSlot(slotID)
			break; -- Continue, because we will break the repeat loop.
		end

		-- Don't even do anything else if we're not the masterlooter
		--if not playerIsLootmaster then break end;

		-- Just go to the next lootslot if this one doesn't contain any items
		if not LootSlotIsItem(slotID) then break end;
		
		-- We're looting an item, so lets get some info about the item		
		local lootIcon, lootName, lootQuantity, rarity, locked, loot1, loot2 = GetLootSlotInfo(slotID);
		

		--[[ We're only gonna bug the raid about this loot if it matches the threshold because
		     the masterlooter can only distribute itemloot that matches this threshold.

		     The threshold is	* 0 - Poor
    					* 1 - Common
					* 2 - Uncommon
					* 3 - Rare
					* 4 - Epic
					* 5 - Legendary
					* 6 - Artifact
		]]		
		--if ( (rarity<2) or (rarity < GetLootThreshold()) ) then break end		

		-- A little sanity check; lets see if quantity == 1
		if lootQuantity~=1 then
			self:Print( format("Could not redistribute %s because quantity != 1 (%s). Please do it manually.", link, lootQuantity) )
			break;
		end

		-- Ok, now we have everything, lets cache the loot
		self:SetLootTableSlotLink( nil, slotID, link )

		

		

		local itemBind = LootMaster:GetItemBinding( link )

		self:Print( format("%s, %s, %s, %s, %s, %s, locked=%s, l1=%s, l2=%s", lootIcon or "nil", lootName or "nil", lootQuantity or "nil", rarity or "nil", link or "nil", itemBind or "not binding", locked or 'nil', loot1 or 'nil', loot2 or 'nil') )

		--if rarity >= GetLootThreshold() then
		--	LootSlot(slotID)
		--end

		-- Lets find out how many peeps in the group are able to receive the loot
		for iCandidate = 1, 40 do repeat
			local candidate = GetMasterLootCandidate(iCandidate)
			if not candidate then break end;

			self:Print( format("    - candidate (%s) %s", iCandidate, candidate) )
	
			if ((not itemBind) or (itemBind~='pickup')) then
			--local ret = GiveMasterLoot(slotID, iCandidate) 

			--self:Print( format("      - GiveMasterLoot ret: %s", tostring(ret)) )
			end
		until true end


		--local lootThresholdStr = getglobal("ITEM_QUALITY"..GetLootThreshold().."_DESC")


	until true; end

	self:Print("End of loot")

	
end
